"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs-extra");
var apiRequest_1 = require("./apiRequest");
var Csv_1 = require("./Csv");
var Excel_1 = require("./Excel");
var labels = {
    green: '5adee9f4841642c2a8e47bc3',
    red: '5adee9f4841642c2a8e47bc4',
    yellow: '5adee9f4841642c2a8e47bc1',
};
var urlAttachmentsTemplate = 'https://api.trello.com/1/cards/{cardId}/attachments';
var urlPutTemplate = 'https://api.trello.com/1/cards/{cardId}';
var urlPost = 'https://api.trello.com/1/cards';
var urlCardsTemplate = 'https://api.trello.com/1/boards/{boardId}/cards';
var urlListsTemplate = 'https://api.trello.com/1/boards/{boardId}/lists';
function getTrelloBoardInfo(boardId, trelloKey, trelloToken) {
    return new Promise(function (resolve, reject) {
        var finished = false;
        var data = {
            lists: [],
            cards: [],
        };
        var urlCards = urlCardsTemplate.replace('{boardId}', boardId);
        var urlLists = urlListsTemplate.replace('{boardId}', boardId);
        var qs = {
            key: trelloKey,
            token: trelloToken,
        };
        apiRequest_1.getRequest(urlLists, qs)
            .then(function (body) {
            if (finished) {
                data.lists = body;
                resolve(data);
            }
            else {
                finished = true;
                data.lists = body;
            }
        })
            .catch(function (err) {
            if (!finished) {
                finished = true;
                reject(err);
            }
        });
        apiRequest_1.getRequest(urlCards, qs)
            .then(function (body) {
            if (finished) {
                data.cards = body;
                resolve(data);
            }
            else {
                finished = true;
                data.cards = body;
            }
        })
            .catch(function (err) {
            if (!finished) {
                finished = true;
                reject(err);
            }
        });
    });
}
exports.getTrelloBoardInfo = getTrelloBoardInfo;
function getDataFromTrello(config) {
    return new Promise(function (resolve, reject) {
        getTrelloBoardInfo(config.trelloBoardId, config.trelloKey, config.trelloToken)
            .then(function (data) {
            var _loop_1 = function (i) {
                if (data.cards[i].name.toLowerCase() === config.trelloCardName.toLowerCase()) {
                    for (var j = 0; j < data.lists.length; j++) {
                        if (data.lists[j].name.toLowerCase() === 'validate' && data.lists[j].id === data.cards[i].idList) {
                            var urlAttachments = urlAttachmentsTemplate.replace('{cardId}', data.cards[i].id);
                            apiRequest_1.getRequest(urlAttachments, { key: config.trelloKey, token: config.trelloToken })
                                .then(function (files) {
                                if (files.length > 0) {
                                    var d = new Date(files[0].date);
                                    var index = 0;
                                    for (var f = 0; f < files.length; f++) {
                                        var t = new Date(files[f].date);
                                        if (d.getTime() < t.getTime()) {
                                            index = f;
                                            d = t;
                                        }
                                    }
                                    if (files[index].name.search('.csv') !== -1 &&
                                        config.trelloFileName === files[index].name.split('.csv')[0]) {
                                        downloadAttachment(config, files[index].url, files[index].name, data.cards[i].id)
                                            .then(function (filePath) {
                                            Csv_1.getDataFromCsv(filePath, config.structure)
                                                .then(function (res) {
                                                var checkedData = dataCheck(res, config.structure);
                                                if (typeof checkedData === 'string') {
                                                    moveCardInErrorList(data, config, checkedData, data.cards[i].id)
                                                        .then(function () {
                                                        reject();
                                                    })
                                                        .catch(function () {
                                                        reject();
                                                    });
                                                }
                                                else {
                                                    moveCardInSuccessList(data, config, data.cards[i].id);
                                                    resolve(checkedData);
                                                }
                                            })
                                                .catch(function (err) {
                                                reject();
                                            });
                                        })
                                            .catch(function (err) {
                                            reject();
                                        });
                                    }
                                    else if (files[index].name.search('.xlsx') !== -1 &&
                                        config.trelloFileName === files[index].name.split('.xlsx')[0]) {
                                        downloadAttachment(config, files[index].url, files[index].name, data.cards[i].id)
                                            .then(function (filePath) {
                                            var res = Excel_1.readXlscFile(filePath);
                                            var checkedData = dataCheck(res, config.structure);
                                            if (typeof checkedData === 'string') {
                                                moveCardInErrorList(data, config, checkedData, data.cards[i].id)
                                                    .then(function () {
                                                    reject();
                                                })
                                                    .catch(function () {
                                                    reject();
                                                });
                                            }
                                            else {
                                                moveCardInSuccessList(data, config, data.cards[i].id);
                                                resolve(checkedData);
                                            }
                                        })
                                            .catch(function (err) {
                                            reject();
                                        });
                                    }
                                }
                                else {
                                    createNewCard(data, config)
                                        .then(function () {
                                        resolve(null);
                                    })
                                        .catch(function (err) {
                                        reject();
                                    });
                                }
                            })
                                .catch(function (err) {
                                // tslint:disable-next-line:no-console
                                console.error(err);
                                reject();
                            });
                        }
                        else if (data.lists[j].name.toLowerCase() === 'validate') {
                            // no change
                            resolve(null);
                            break;
                        }
                    }
                    return "break";
                }
                else if (i === data.cards.length - 1) {
                    // create card
                    createNewCard(data, config)
                        .then(function () {
                        resolve({ change: false });
                    })
                        .catch(function (err) {
                        reject();
                    });
                }
            };
            for (var i = 0; i < data.cards.length; i++) {
                var state_1 = _loop_1(i);
                if (state_1 === "break")
                    break;
            }
        })
            .catch(function (err) {
            // tslint:disable-next-line:no-console
            console.error(err);
            reject();
        });
    });
}
exports.getDataFromTrello = getDataFromTrello;
function downloadAttachment(config, url, name, cardId) {
    return new Promise(function (resolve, reject) {
        var qs = {
            key: config.trelloKey,
            token: config.trelloToken,
            desc: '#Validation in progress',
            idLabels: labels.yellow,
        };
        var urlPut = urlPutTemplate.replace('{cardId}', cardId);
        apiRequest_1.putRequest(urlPut, qs).catch(function (err) {
            // tslint:disable-next-line:no-console
            console.error(err);
        });
        var downloadObj = {
            directory: config.storageDirectory,
            name: name,
            url: url,
            id: '0',
            headers: {
                Authorization: 'OAuth oauth_consumer_key="' + config.trelloKey + '", oauth_token="' + config.trelloToken + '"',
            },
        };
        apiRequest_1.downloadFileWithTrelloHeader(downloadObj)
            .then(function (res) {
            fs.writeFileSync(res.dir, res.data);
            resolve(res.dir);
        })
            .catch(function (err) {
            // tslint:disable-next-line:no-console
            console.log(err);
            reject(err);
        });
    });
}
exports.downloadAttachment = downloadAttachment;
function moveCardInSuccessList(data, config, cardId) {
    return new Promise(function (resolve, reject) {
        for (var j = 0; j < data.lists.length; j++) {
            if (data.lists[j].name.toLowerCase() === 'success') {
                var d = new Date();
                var qs = {
                    key: config.trelloKey,
                    token: config.trelloToken,
                    desc: '#Validation successful (' + d.toUTCString() + ')',
                    idLabels: labels.green,
                    idList: data.lists[j].id,
                };
                var urlPut = urlPutTemplate.replace('{cardId}', cardId);
                apiRequest_1.putRequest(urlPut, qs)
                    .then(function () {
                    resolve({});
                })
                    .catch(function (err) {
                    // tslint:disable-next-line:no-console
                    console.error(err);
                    reject(err);
                });
                break;
            }
        }
    });
}
exports.moveCardInSuccessList = moveCardInSuccessList;
function moveCardInErrorList(data, config, msg, cardId) {
    return new Promise(function (resolve, reject) {
        for (var j = 0; j < data.lists.length; j++) {
            if (data.lists[j].name.toLowerCase() === 'error') {
                var qs = {
                    key: config.trelloKey,
                    token: config.trelloToken,
                    desc: msg,
                    idLabels: labels.red,
                    idList: data.lists[j].id,
                };
                var urlPut = urlPutTemplate.replace('{cardId}', cardId);
                apiRequest_1.putRequest(urlPut, qs)
                    .then(function () {
                    resolve({});
                })
                    .catch(function (err) {
                    // tslint:disable-next-line:no-console
                    console.error(err);
                    reject(err);
                });
            }
        }
    });
}
exports.moveCardInErrorList = moveCardInErrorList;
function createNewCard(data, config) {
    return new Promise(function (resolve, reject) {
        for (var j = 0; j < data.lists.length; j++) {
            if (data.lists[j].name.toLowerCase() === 'error') {
                var qs = {
                    key: config.trelloKey,
                    token: config.trelloToken,
                    desc: '#Created new card',
                    idLabels: labels.red,
                    idList: data.lists[j].id,
                    name: config.trelloCardName,
                };
                apiRequest_1.postRequest(urlPost, qs)
                    .then(function () {
                    resolve(true);
                })
                    .catch(function (err) {
                    // tslint:disable-next-line:no-console
                    console.error(err);
                    reject(err);
                });
                break;
            }
            else if (data.lists.length - 1 === j) {
                reject();
            }
        }
    });
}
exports.createNewCard = createNewCard;
function dataCheck(data, schema) {
    var newJson = [];
    for (var i = 0; i < data.length; i++) {
        for (var j = 0; j < schema.length; j++) {
            if (data[i][j] === undefined) {
                return '#Error in file. Missing value (' + schema[j].name + '). Error probably in line ' + (i + 1).toString();
            }
        }
        if (data[i].length !== schema.length) {
            return '#Error in file. Missing values. Error probably in line ' + (i + 1).toString();
        }
    }
    for (var n = 0; n < schema.length; n++) {
        if (data[0][n].search(schema[n].name) === -1) {
            return '#Error in file. Problem with ' + schema[n] + '. Error probably in line 1';
        }
        if (!schema[n].empty && schema[n].empty !== undefined) {
            for (var i = 0; i < data.length; i++) {
                if (!data[i][n].toString()) {
                    return ('#Error in file. Empty' + data[0][n].toUpperCase() + ' found. Error probably in line ' + (i + 1).toString());
                }
            }
        }
        if (schema[n].unique) {
            for (var i = 1; i < data.length; i++) {
                for (var j = 1; j < data.length; j++) {
                    if (data[i][n] === data[j][n] && i !== j) {
                        return ('#Error in file. ' +
                            schema[n].name.toUpperCase() +
                            ' received: ( ' +
                            data[i][n] +
                            ' ) is not unique. Error probably in line ' +
                            (i + 1).toString() +
                            ' and ' +
                            (j + 1).toString());
                    }
                }
            }
        }
        if (schema[n].number) {
            for (var i = 1; i < data.length; i++) {
                if (isNaN(data[i][n])) {
                    return ('#Error in file. Received: ( ' +
                        data[i][n] +
                        ' ). Expected numeric value for ' +
                        schema[n].name.toUpperCase() +
                        '. Error probably in line ' +
                        (i + 1).toString());
                }
            }
        }
        if (schema[n].number !== undefined && schema[n].number && schema[n].range !== null) {
            for (var i = 1; i < data.length; i++) {
                if (parseFloat(data[i][n]) < schema[n].range[0] || parseFloat(data[i][n]) > schema[n].range[1]) {
                    return ('#Error in file. Value received: ( ' +
                        parseFloat(data[i][n]) +
                        ' ) for ' +
                        schema[n].name.toUpperCase() +
                        ' is out of range. Error probably in line ' +
                        (i + 1).toString());
                }
            }
        }
        if (schema[n].requiredType) {
            for (var i = 1; i < data.length; i++) {
                if (schema[n].types.indexOf(data[i][n].toLowerCase()) === -1) {
                    return ('#Error in file. ' +
                        schema[n].name.toUpperCase() +
                        ' received : ( ' +
                        data[i][n] +
                        ' ) type is not avaialable. Error probably in line ' +
                        (i + 1).toString());
                }
            }
        }
        if (schema[n].path !== undefined && schema[n].path) {
            for (var i = 1; i < data.length; i++) {
                if (data[i][n].search(/[<>|*?]/g) !== -1) {
                    return ('#Error in file. ' +
                        schema[n].name.toUpperCase() +
                        ' received : ( ' +
                        data[i][n] +
                        ' ) contains unallowed characters: [<>|*?:]. Error probably in line ' +
                        (i + 1).toString());
                }
            }
        }
    }
    var dataString = JSON.stringify(data);
    if (dataString.indexOf('\ufffd') !== -1) {
        return '#Error in .csv file. Problem with encoding. Must be UTF-8.';
    }
    var _loop_2 = function (i) {
        var newItem = {};
        schema.forEach(function (head, index) {
            if (data[i][index] !== undefined) {
                if (head.number !== undefined && head.number && head.range !== null) {
                    newItem[head.name] = parseFloat(data[i][index]);
                }
                else if (head.path !== undefined && head.path) {
                    var dest = data[i][index].replace(/\\/g, '/');
                    if (dest.charAt(dest.length - 1) !== '/') {
                        dest += '/';
                    }
                    dest = dest.replace(/ \//g, '/');
                    newItem[head.name] = dest;
                }
                else {
                    if (typeof data[i][index] === 'string') {
                        newItem[head.name] = data[i][index];
                    }
                    else {
                        newItem[head.name] = data[i][index].toString();
                    }
                }
            }
            else {
                newItem[head.name] = null;
            }
        });
        newJson.push(newItem);
    };
    for (var i = 1; i < data.length; i++) {
        _loop_2(i);
    }
    return newJson;
}
exports.dataCheck = dataCheck;
var DataModel = /** @class */ (function () {
    function DataModel() {
    }
    return DataModel;
}());
exports.DataModel = DataModel;
//# sourceMappingURL=Trello.js.map