"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs-extra");
var Hash_1 = require("./Hash");
var VersionModel = /** @class */ (function () {
    function VersionModel() {
    }
    return VersionModel;
}());
exports.VersionModel = VersionModel;
function exists(id, storageDirectory) {
    if (storageDirectory !== undefined) {
        var directory = storageDirectory + 'version_files/';
        if (fs.existsSync(directory + id + '.json')) {
            return true;
        }
        else {
            return false;
        }
    }
    return false;
}
exports.exists = exists;
function getAllFiles(storageDirectory) {
    if (storageDirectory !== undefined) {
        var directory = storageDirectory + 'version_files/';
        return fs.readdirSync(directory);
    }
    else {
        throw {
            name: 'storageDirectory undefined',
            message: 'Version.ts / getAllFiles()',
        };
    }
}
exports.getAllFiles = getAllFiles;
function forgetAll(storageDirectory) {
    if (storageDirectory !== undefined) {
        fs.removeSync(storageDirectory + 'version_files/');
        fs.removeSync(storageDirectory + 'unsupported/');
        return true;
    }
    return false;
}
exports.forgetAll = forgetAll;
function removeItem(id, storageDirectory) {
    if (storageDirectory !== undefined) {
        var directory = storageDirectory + 'version_files/';
        if (fs.existsSync(directory + id + '.json')) {
            fs.unlinkSync(directory + id + '.json');
        }
        return true;
    }
    return false;
}
exports.removeItem = removeItem;
function lastSeen(id, storageDirectory) {
    if (storageDirectory !== undefined) {
        var directory = storageDirectory + 'version_files/';
        if (fs.existsSync(directory + id + '.json')) {
            var obj = JSON.parse(fs.readFileSync(directory + id + '.json', 'utf8'));
            return obj.lastSeen;
        }
        else {
            throw {
                name: 'file undefined',
                message: 'Version.ts / lastSeen()',
            };
        }
    }
    else {
        throw {
            name: 'storageDirectory undefined',
            message: 'Version.ts / lastSeen()',
        };
    }
}
exports.lastSeen = lastSeen;
function lastChange(id, storageDirectory) {
    if (storageDirectory !== undefined) {
        var directory = storageDirectory + 'version_files/';
        if (fs.existsSync(directory + id + '.json')) {
            var obj = JSON.parse(fs.readFileSync(directory + id + '.json', 'utf8'));
            return obj.timestamp;
        }
        else {
            throw {
                name: 'file undefined',
                message: 'Version.ts / lastChange()',
            };
        }
    }
    else {
        throw {
            name: 'storageDirectory undefined',
            message: 'Version.ts / lastChange()',
        };
    }
}
exports.lastChange = lastChange;
function checkItem(id, information, storageDirectory) {
    var d = new Date();
    if (storageDirectory !== undefined) {
        var directory = storageDirectory + 'version_files/';
        if (!fs.existsSync(directory)) {
            fs.mkdirSync(directory);
        }
        if (fs.existsSync(directory + id + '.json')) {
            var oldObj = JSON.parse(fs.readFileSync(directory + id + '.json', 'utf8'));
            var newHash = Hash_1.getMd5Hash(information);
            if (newHash === oldObj.hash) {
                oldObj.lastSeen = d.getTime();
                fs.writeFileSync(directory + id + '.json', JSON.stringify(oldObj));
                return {
                    changed: false,
                    new: false,
                };
            }
            else {
                oldObj.hash = newHash;
                oldObj.timestamp = d.getTime();
                oldObj.lastSeen = d.getTime();
                oldObj.version = oldObj.version + 1;
                fs.writeFileSync(directory + id + '.json', JSON.stringify(oldObj));
                return {
                    changed: true,
                    new: false,
                };
            }
        }
        else {
            var obj = {
                hash: Hash_1.getMd5Hash(information),
                timestamp: d.getTime(),
                lastSeen: d.getTime(),
                version: 1,
            };
            fs.writeFileSync(directory + id + '.json', JSON.stringify(obj));
            return {
                changed: true,
                new: true,
            };
        }
    }
    else {
        throw {
            name: 'storageDirectory undefined',
            message: 'Version.ts / checkItem()',
        };
    }
}
exports.checkItem = checkItem;
//# sourceMappingURL=Version.js.map