"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs-extra");
var apiRequest_1 = require("./apiRequest");
function getElevation(parameters, config) {
    return new Promise(function (resolve, reject) {
        if (!fs.existsSync(config.storageDirectory + 'rawData')) {
            fs.mkdirSync(config.storageDirectory + 'rawData');
        }
        if (!fs.existsSync(config.storageDirectory + 'rawData/' + config.tessellation)) {
            fs.mkdirSync(config.storageDirectory + 'rawData/' + config.tessellation);
        }
        var fileCoords = [0, 0];
        if (parameters.lat < 0) {
            fileCoords[1] = Math.floor(((90 + parameters.lat) * config.tessellation) / 90);
        }
        else {
            fileCoords[1] = Math.floor(((90 + parameters.lat) * config.tessellation) / 90);
        }
        if (parameters.lon < 0) {
            fileCoords[0] = Math.floor(((360 + parameters.lon) * config.tessellation) / 90);
        }
        else {
            fileCoords[0] = Math.floor((parameters.lon * config.tessellation) / 90);
        }
        if (fs.existsSync(config.storageDirectory +
            'rawData/' +
            config.tessellation +
            '/' +
            fileCoords[0] +
            '/' +
            fileCoords[1] +
            '.json')) {
            resolve(calculateElevation(parameters.lon, parameters.lat, config.tessellation, config.storageDirectory));
        }
        else {
            if (!fs.existsSync(config.storageDirectory + 'rawData/' + config.tessellation + '/' + fileCoords[0])) {
                fs.mkdirSync(config.storageDirectory + 'rawData/' + config.tessellation + '/' + fileCoords[0]);
            }
            var downloadObj = {
                url: config.terrainData +
                    (config.tessellation * 2).toString() +
                    '/' +
                    fileCoords[0] +
                    '/' +
                    fileCoords[1] +
                    '.json',
                id: '1',
                directory: config.storageDirectory + 'rawData/' + config.tessellation + '/' + fileCoords[0] + '/',
                name: fileCoords[1] + '.json',
            };
            apiRequest_1.downloadFile(downloadObj)
                .then(function (res) {
                fs.writeFileSync(res.dir, res.data);
                resolve(calculateElevation(parameters.lon, parameters.lat, config.tessellation, config.storageDirectory));
            })
                .catch(function (err) {
                reject();
            });
        }
    });
}
exports.getElevation = getElevation;
function calculateElevation(lon, lat, tessellation, storageDirectory) {
    var fileCoords = [0, 0];
    if (lat < 0) {
        fileCoords[1] = ((90 + lat) * tessellation) / 90;
    }
    else {
        fileCoords[1] = ((90 + lat) * tessellation) / 90;
    }
    if (lon < 0) {
        fileCoords[0] = ((360 + lon) * tessellation) / 90;
    }
    else {
        fileCoords[0] = (lon * tessellation) / 90;
    }
    var fileIn = storageDirectory +
        'rawData/' +
        tessellation +
        '/' +
        Math.floor(fileCoords[0]).toString() +
        '/' +
        Math.floor(fileCoords[1]).toString() +
        '.json';
    var fileData = fs.readFileSync(fileIn, 'utf8');
    if (fileData.startsWith(String.fromCharCode(65279))) {
        fileData = fileData.substring(1, fileData.length);
    }
    fileData = JSON.parse(fileData);
    var bottomLeft = [
        (fileCoords[0] - Math.floor(fileCoords[0])) * (fileData.length - 1),
        (fileCoords[1] - Math.floor(fileCoords[1])) * (fileData.length - 1),
    ];
    var bou = [
        fileData[Math.floor(bottomLeft[1])][Math.floor(bottomLeft[0])],
        fileData[Math.floor(bottomLeft[1])][Math.ceil(bottomLeft[0])],
        fileData[Math.ceil(bottomLeft[1])][Math.ceil(bottomLeft[0])],
        fileData[Math.ceil(bottomLeft[1])][Math.floor(bottomLeft[0])],
    ];
    var z1 = bou[0] * (1 - (bottomLeft[0] - Math.floor(bottomLeft[0]))) + bou[1] * (bottomLeft[0] - Math.floor(bottomLeft[0]));
    var z2 = bou[3] * (1 - (bottomLeft[0] - Math.floor(bottomLeft[0]))) + bou[2] * (bottomLeft[0] - Math.floor(bottomLeft[0]));
    var z = z1 * (1 - (bottomLeft[1] - Math.floor(bottomLeft[1]))) + z2 * (bottomLeft[1] - Math.floor(bottomLeft[1]));
    return Math.round(z);
}
//# sourceMappingURL=getElevation.js.map